#pragma once
#include "libs.h"
#include "rigid_body.h"
#include "ship_type.h"
#include "sbre/sbre.h"

class SpaceStation;

struct shipstats_t {
  int max_capacity;
  int used_capacity;
  int free_capacity;
  int total_mass; /* Cargo, equipment + hull. */
  float hyperspace_range;
};

class Ship : public RigidBody {
public:
  Ship(ShipType::Type shipType);
  virtual void AITurn(void);
  virtual Object::Type GetType(void) { return Object::SHIP; }
  virtual void SetDockedWith(SpaceStation*);
  SpaceStation* GetDockedWith(void) { return m_dockedWith; }
  void SetTarget(Body* const target) { m_target = target; }
  Body* GetTarget(void) const { return m_target; }
  virtual void Render(const Frame* camFrame);
  void SetThrusterState(enum ShipType::Thruster t, float level);
  void SetAngThrusterState(int axis, float level) { m_angThrusters[axis] = CLAMP(level, -1, 1); }
  void ClearThrusterState(void);
  void SetGunState(int idx, int state);
  const ShipType& GetShipType(void);
  void CalcStats(shipstats_t* stats);
  void UpdateMass(void);
  void SetWheelState(bool down);

  class LaserObj : public Object {
  public:
    virtual Object::Type GetType(void) { return Object::LASER; }
    Ship* owner;
  };

  EquipSet m_equipment;
protected:
  void RenderLaserfire(void);
  
  SpaceStation* m_dockedWith;
  enum ShipType::Type m_shipType;
  Uint32 m_gunState[ShipType::GUNMOUNT_MAX];
private:
  float m_wheelState;
  float m_wheelTransition;

  float m_thrusters[ShipType::THRUSTER_MAX];
  float m_angThrusters[3];
  dGeomID m_tempLaserGeom[ShipType::GUNMOUNT_MAX];

  LaserObj m_laserCollisionObj;
  Body* m_target;
};

